<?php
class Zpayment_Zpayment_Model_Payment extends Mage_Payment_Model_Method_Abstract
{
    protected $_isGateway               = true;
    protected $_canUseInternal          = false;
    protected $_canUseForMultishipping  = false;

    protected $_code = 'zpayment';

    private $_order;
    private $_ipnData = array();
    private $_ipnValidator;

    /**
     * Возвращает URL для перенаправления покупателя после создания заказа.
     *
     * @access public
     * @return string
     */
    public function getOrderPlaceRedirectUrl()
    {
        return Mage::getUrl('zpayment/index/redirect', array('_secure' => true));
    }

    /**
     * Возвращает данные для перенаправления на Zpayment.
     *
     * @access public
     * @return array
     */
    public function getRedirectFormFields()
    {
        $result = array();

        $order = $this->getOrder();
        if (!$order->getId()) {
            return $result;
        }

        $result['shopId']    = Mage::getStoreConfig('payment/zpayment/zpayment_shop_id');
		$result['initPass']  = Mage::getStoreConfig('payment/zpayment/zpayment_initpass');
		$result['merchKey']  = Mage::getStoreConfig('payment/zpayment/zpayment_secret');
        $result['userEmail'] = $order->getCustomerEmail();
        $result['amount']    = $order->getGrandTotal();
        $result['currency']  = strtolower($order->getOrderCurrencyCode());
        $result['paymentId'] = $order->getIncrementId();
        $result['purpose']   = 'Order id #'.$result['paymentId'];
        $result['src'] = '1';
        $language = strtolower(Mage::app()->getLocale()->getLocaleCode());
        $result['language']  = ($language == 'ru') ? $language : 'en';

        return $result;
    }

    /**
     * Валидация оповещения о платеже от Zpayment.
     *
     * @access public
     * @return mixed
     */
    public function validateIpn($request)
    {
		foreach ($request as $Key=>$Value) { $$Key = $Value; }
		$model = Mage::getModel('zpayment/payment');
        $data = $this->getRedirectFormFields();
		$merch_key = Mage::getStoreConfig('payment/zpayment/zpayment_secret');
		$shopId = Mage::getStoreConfig('payment/zpayment/zpayment_shop_id');
		$err = true;
		if ($shopId!=$LMI_PAYEE_PURSE) $err = false;
		$order = Mage::getModel('sales/order')->loadByIncrementId($LMI_PAYMENT_NO);
        if (!$order){ $err = false; }
		$summ = number_format($order->getGrandTotal(), 2, '.', '');
		if ($summ!=$LMI_PAYMENT_AMOUNT){ $err = false; }

		if(isset($LMI_SECRET_KEY)) {
            if($LMI_SECRET_KEY!=$merch_key) {
                $err = false;
			}
		} else {
            $CalcHash = md5($LMI_PAYEE_PURSE.$LMI_PAYMENT_AMOUNT.$LMI_PAYMENT_NO.$LMI_MODE.$LMI_SYS_INVS_NO.$LMI_SYS_TRANS_NO.$LMI_SYS_TRANS_DATE.$merch_key.$LMI_PAYER_PURSE.$LMI_PAYER_WM);
		     if($LMI_HASH != strtoupper($CalcHash)) {
                $err = false;
		    }
        }
        return $err;
    }


    /**
     * Возвращает текущий заказ.
     *
     * @access public
     * @return mixed
     */
    public function getOrder()
    {
        if (!$this->_order) {
            $session = Mage::getSingleton('checkout/session');
            $this->_order = $this->loadOrderById($session->getLastRealOrderId());
        }
        return $this->_order;
    }

    /**
     * Загружает заказ по идентификатору.
     *
     * @param int $orderId
     * @access private
     * @return mixed
     */
    private function loadOrderById($orderId)
    {
        return Mage::getModel('sales/order')->loadByIncrementId($orderId);
    }

    /**
     * Возвращает данные пришедшие в оповещении о платеже.
     *
     * @access public
     * @return array
     */
    public function getIpnData()
    {
        return $this->_ipnData;
    }

    /**
     * Возвращает объект для валидации оповещения о платеже.
     *
     * @access private
     * @return ZpaymentNotificationValidator
     */
    private function getIpnValidator()
    {
        if (!$this->_ipnValidator) {
            $file = Mage::getBaseDir().DS.'lib'.DS.'zpaymentlib'.DS.'ZpaymentNotificationValidator.php';
            if (file_exists($file))
                require_once($file);
            else
                throw new Exception('Zpaymentlib: class '.$className.' not found');
            $this->_ipnValidator = new ZpaymentNotificationValidator();
        }
        return $this->_ipnValidator;
    }
}
